@@ -7,7 +7,7 @@
  *      implementation of the pqxx::connection_base abstract base class.
  *   pqxx::connection_base encapsulates a frontend to backend connection
  *
- * Copyright (c) 2001-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2009, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -49,9 +49,14 @@
 #include "pqxx/nontransaction"
 #include "pqxx/pipeline"
 #include "pqxx/result"
+#include "pqxx/strconv"
 #include "pqxx/transaction"
 #include "pqxx/notify-listen"
 
+#include "pqxx/internal/gates/result-creation.hxx"
+#include "pqxx/internal/gates/result-connection.hxx"
+#include "pqxx/internal/gates/connection-reactivation_avoidance_exemption.hxx"
+
 using namespace PGSTD;
 using namespace pqxx;
 using namespace pqxx::internal;
@@ -111,7 +116,7 @@ static void clear_fdmask(fd_set *mask)
 string pqxx::encrypt_password(const string &user, const string &password)
 {
   PQAlloc<char> p(PQencryptPassword(password.c_str(), user.c_str()));
-  return string(p.c_ptr());
+  return string(p.get());
 }
 #endif
 
@@ -119,16 +124,16 @@ string pqxx::encrypt_password(const string &user, const string &password)
 pqxx::connection_base::connection_base(connectionpolicy &pol) :
   m_Conn(0),
   m_policy(pol),
-  m_Completed(false),
   m_Trans(),
   m_Noticer(),
   m_defaultNoticeProcessor(0),
   m_Trace(0),
   m_serverversion(0),
-  m_caps(),
-  m_inhibit_reactivation(false),
   m_reactivation_avoidance(),
-  m_unique_id(0)
+  m_unique_id(0),
+  m_Completed(false),
+  m_inhibit_reactivation(false),
+  m_caps()
 {
   clearcaps();
 }
@@ -141,6 +146,18 @@ void pqxx::connection_base::init()
 }
 
 
+pqxx::result pqxx::connection_base::make_result(
+	internal::pq::PGresult *rhs,
+	const PGSTD::string &query)
+{
+  return gate::result_creation::create(
+	rhs,
+	protocol_version(),
+	query,
+	encoding_code());
+}
+
+
 int pqxx::connection_base::backendpid() const throw ()
 {
   return m_Conn ? PQbackendPID(m_Conn) : 0;
@@ -281,7 +298,7 @@ string pqxx::connection_base::RawGetVar(const PGSTD::string &Var)
 
 void pqxx::connection_base::clearcaps() throw ()
 {
-  for (int i=0; i<cap_end; ++i) m_caps[i] = false;
+  m_caps.reset();
 }
 
 
@@ -343,11 +360,9 @@ void pqxx::connection_base::SetupState()
     // Now do the whole batch at once
     PQsendQuery(m_Conn, restore_query.str().c_str());
     result r;
-    const int proto = protocol_version();
-    const int encoding = encoding_code();
     do
-      r = result(PQgetResult(m_Conn), proto, "[RECONNECT]", encoding);
-    while (r);
+      r = make_result(PQgetResult(m_Conn), "[RECONNECT]");
+    while (gate::result_connection(r));
   }
 
   m_Completed = true;
@@ -360,11 +375,11 @@ void pqxx::connection_base::check_result(const result &R)
   if (!is_open()) throw broken_connection();
 
   // A shame we can't quite detect out-of-memory to turn this into a bad_alloc!
-  if (!R) throw failure(ErrMsg());
+  if (!gate::result_connection(R)) throw failure(ErrMsg());
 
   try
   {
-    R.CheckStatus();
+    gate::result_creation(R).CheckStatus();
   }
   catch (const exception &e)
   {
@@ -586,12 +601,7 @@ void pqxx::connection_base::add_listener(pqxx::notify_listener *T)
 
     if (is_open()) try
     {
-      result R( PQexec(m_Conn,
-		LQ.c_str()),
-		protocol_version(),
-		LQ,
-		encoding_code() );
-      check_result(R);
+      check_result(make_result(PQexec(m_Conn, LQ.c_str()), LQ));
     }
     catch (const broken_connection &)
     {
@@ -652,41 +662,70 @@ bool pqxx::connection_base::is_busy() const throw ()
 }
 
 
-#ifdef PQXX_HAVE_PQCANCEL
 namespace
 {
 class cancel_wrapper
 {
+#ifdef PQXX_HAVE_PQCANCEL
   PGcancel *m_cancel;
   char m_errbuf[500];
+#else
+  PGconn *m_conn;
+#endif
+
 public:
   cancel_wrapper(PGconn *conn) :
+#ifdef PQXX_HAVE_PQCANCEL
     m_cancel(NULL),
     m_errbuf()
+#else
+    m_conn(conn)
+#endif
   {
-    m_cancel = PQgetCancel(conn);
-    if (!m_cancel) throw bad_alloc();
+    if (conn)
+    {
+#ifdef PQXX_HAVE_PQCANCEL
+      m_cancel = PQgetCancel(conn);
+      if (!m_cancel) throw bad_alloc();
+#endif
+    }
   }
-  ~cancel_wrapper() { PQfreeCancel(m_cancel); }
+#ifdef PQXX_HAVE_PQCANCEL
+  ~cancel_wrapper() { if (m_cancel) PQfreeCancel(m_cancel); }
+#endif
 
   void operator()()
   {
-    if (!PQcancel(m_cancel, m_errbuf, int(sizeof(m_errbuf))))
+#ifdef PQXX_HAVE_PQCANCEL
+    if (m_cancel && !PQcancel(m_cancel, m_errbuf, int(sizeof(m_errbuf))))
       throw sql_error(string(m_errbuf));
+#else
+    if (m_conn && !PQrequestCancel(m_conn))
+      throw sql_error(PQerrorMessage(m_conn));
+#endif
   }
 };
 }
-#endif
+
 
 void pqxx::connection_base::cancel_query()
 {
-#ifdef PQXX_HAVE_PQCANCEL
   cancel_wrapper cancel(m_Conn);
   cancel();
+}
+
+
+namespace
+{
+void freemem_notif(PGnotify *p) throw ()
+{
+#ifdef PQXX_HAVE_PQFREENOTIFY
+  PQfreeNotify(p);
 #else
-  if (!PQrequestCancel(m_Conn)) throw sql_error(ErrMsg());
+  freepqmem(p);
 #endif
 }
+} // namespace
 
 
 int pqxx::connection_base::get_notifs()
@@ -700,7 +739,10 @@ int pqxx::connection_base::get_notifs()
   // deliver them.
   if (m_Trans.get()) return notifs;
 
-  for (PQAlloc<PGnotify> N( PQnotifies(m_Conn) ); N; N = PQnotifies(m_Conn))
+  typedef PQAlloc<PGnotify, freemem_notif> notifptr;
+  for (notifptr N( PQnotifies(m_Conn) );
+       N.get();
+       N = notifptr(PQnotifies(m_Conn)))
   {
     typedef listenerlist::iterator TI;
 
@@ -734,7 +776,7 @@ int pqxx::connection_base::get_notifs()
       }
     }
 
-    N.clear();
+    N.reset();
   }
   return notifs;
 }
@@ -778,17 +820,13 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 {
   activate();
 
-  result R(PQexec(m_Conn, Query), protocol_version(), Query, encoding_code());
+  result R = make_result(PQexec(m_Conn, Query), Query);
 
-  while ((Retries > 0) && !R && !is_open())
+  while ((Retries > 0) && !gate::result_connection(R) && !is_open())
   {
     Retries--;
     Reset();
-    if (is_open())
-      R = result(PQexec(m_Conn, Query),
-	protocol_version(),
-	Query,
-	encoding_code());
+    if (is_open()) R = make_result(PQexec(m_Conn, Query), Query);
   }
 
   check_result(R);
@@ -806,11 +844,23 @@ pqxx::prepare::declaration pqxx::connection_base::prepare(
   if (i != m_prepared.end())
   {
     if (definition != i->second.definition)
-      throw argument_error("Inconsistent redefinition "
-	  "of prepared statement " + name);
+    {
+      if (!name.empty())
+        throw argument_error(
+		"Inconsistent redefinition of prepared statement " + name);
 
-    // Prepare for repeated repetition of parameters
+      if (!supports(cap_prepare_unnamed_statement))
+        throw feature_not_supported(
+		"Defining unnamed prepared statements requires a newer "
+		"libpq version.");
+
+      i->second.registered = false;
+      i->second.definition = definition;
+    }
+
+    // Prepare for new definition of parameters
     i->second.parameters.clear();
+    i->second.varargs = false;
     i->second.complete = false;
   }
   else
@@ -818,7 +868,14 @@ pqxx::prepare::declaration pqxx::connection_base::prepare(
     m_prepared.insert(make_pair(name,
 	  prepare::internal::prepared_def(definition)));
   }
-  return prepare::declaration(*this,name);
+  return prepare::declaration(*this, name);
+}
+
+
+pqxx::prepare::declaration pqxx::connection_base::prepare(
+	const PGSTD::string &definition)
+{
+  return this->prepare(string(), definition);
 }
 
 
@@ -902,10 +959,31 @@ void pqxx::connection_base::prepare_param_declare(
     throw usage_error("Attempt to add parameter to prepared statement " +
 	statement +
 	" after its definition was completed");
+  if (s.varargs)
+    throw usage_error("Attempt to add parameters to prepared statement " +
+	statement + " after arbitrary trailing parameters.");
   s.addparam(sqltype,treatment);
 }
 
 
+void pqxx::connection_base::prepare_param_declare_varargs(
+	const PGSTD::string &statement,
+	param_treatment treatment)
+{
+  if (!supports(cap_statement_varargs))
+    throw feature_not_supported(
+	"Prepared statements do not support variable argument lists "
+	"in this configuration.");
+  prepare::internal::prepared_def &s = find_prepared(statement);
+  if (s.complete)
+    throw usage_error("Attempt to add arbitrary parameters to prepared "
+	"statement " + statement + " after its definition was completed.");
+  s.varargs_treatment = treatment;
+  s.varargs = true;
+  s.complete = true;
+}
+ 
+
 pqxx::prepare::internal::prepared_def &
 pqxx::connection_base::register_prepared(const PGSTD::string &name)
 {
@@ -921,12 +999,11 @@ pqxx::connection_base::register_prepared(const PGSTD::string &name)
 #ifdef PQXX_HAVE_PQPREPARE
     if (protocol_version() >= 3)
     {
-      result r(PQprepare(m_Conn, name.c_str(), s.definition.c_str(), 0, 0),
-	protocol_version(),
-	"[PREPARE " + name + "]",
-	encoding_code());
+      result r = make_result(
+	PQprepare(m_Conn, name.c_str(), s.definition.c_str(), 0, 0),
+	"[PREPARE " + name + "]");
       check_result(r);
-      s.registered = true;
+      s.registered = !name.empty();
       return s;
     }
 #endif
@@ -943,7 +1020,7 @@ pqxx::connection_base::register_prepared(const PGSTD::string &name)
 
     P << " AS " << s.definition;
     Exec(P.str().c_str(), 0);
-    s.registered = true;
+    s.registered = !name.empty();
   }
 
   return s;
@@ -964,10 +1041,15 @@ pqxx::result pqxx::connection_base::prepared_exec(
 {
   prepare::internal::prepared_def &s = register_prepared(statement);
 
-  if (nparams != int(s.parameters.size()))
-    throw usage_error("Wrong number of parameters for prepared statement " +
-	statement + ": "
-	"expected " + to_string(s.parameters.size()) + ", "
+  const int expected_params = int(s.parameters.size());
+  if (nparams < expected_params)
+    throw usage_error("Insufficient parameters for prepared statement " +
+	statement + ": expected " + to_string(expected_params) + ", "
+	"received " + to_string(nparams));
+
+  if (nparams > expected_params && !s.varargs)
+    throw usage_error("Too many arguments for prepared statement " +
+	statement + ": expected " + to_string(expected_params) + ", "
 	"received " + to_string(nparams));
 
   result r;
@@ -980,20 +1062,20 @@ pqxx::result pqxx::connection_base::prepared_exec(
     if (protocol_version() >= 3)
     {
       internal::scoped_array<int> binary(nparams+1);
-      for (int i=0; i<nparams; ++i)
+      for (int i=0; i<expected_params; ++i)
         binary[i] = (s.parameters[i].treatment == treat_binary);
+      for (int j=expected_params; j < nparams; ++j)
+        binary[j] = (s.varargs_treatment == treat_binary);
       binary[nparams] = 0;
 
-      r = result(PQexecPrepared(m_Conn,
+      r = make_result(PQexecPrepared(m_Conn,
 		statement.c_str(),
 		nparams,
 		params,
 		paramlengths,
-		binary.c_ptr(),
+		binary.get(),
 		0),
-	protocol_version(),
-	statement,
-	encoding_code());
+	statement);
 
       check_result(r);
       get_notifs();
@@ -1010,7 +1092,8 @@ pqxx::result pqxx::connection_base::prepared_exec(
 	Q << escape_param(*this,
 		params[a],
 		paramlengths[a],
-		s.parameters[a].treatment);
+		(a < expected_params) ?
+			s.parameters[a].treatment : s.varargs_treatment);
 	if (a < nparams-1) Q << ',';
       }
       Q << ')';
@@ -1043,6 +1126,13 @@ pqxx::result pqxx::connection_base::prepared_exec(
 }
 
 
+bool pqxx::connection_base::prepared_exists(const string &statement) const
+{
+  PSMap::const_iterator s = m_prepared.find(statement);
+  return s != PSMap::const_iterator(m_prepared.end());
+}
+
+
 void pqxx::connection_base::Reset()
 {
   if (m_inhibit_reactivation)
@@ -1149,18 +1239,6 @@ void pqxx::connection_base::UnregisterTransaction(transaction_base *T)
 }
 
 
-void pqxx::connection_base::MakeEmpty(pqxx::result &R)
-{
-  if (!m_Conn)
-    throw internal_error("MakeEmpty() on null connection");
-
-  R = result(PQmakeEmptyPGresult(m_Conn, PGRES_EMPTY_QUERY),
-		protocol_version(),
-		"[]",
-		encoding_code());
-}
-
-
 #ifndef PQXX_HAVE_PQPUTCOPY
 namespace
 {
@@ -1179,17 +1257,16 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
 
 #ifdef PQXX_HAVE_PQPUTCOPY
   char *Buf = 0;
-  const int proto = protocol_version(), encoding=encoding_code();
-  const string querydesc = "[END COPY]";
+  const string query = "[END COPY]";
   switch (PQgetCopyData(m_Conn, &Buf, false))
   {
     case -2:
       throw failure("Reading of table data failed: " + string(ErrMsg()));
 
     case -1:
-      for (result R(PQgetResult(m_Conn), proto, querydesc, encoding);
-           R;
-	   R=result(PQgetResult(m_Conn), proto, querydesc, encoding))
+      for (result R(make_result(PQgetResult(m_Conn), query));
+           gate::result_connection(R);
+	   R=make_result(PQgetResult(m_Conn), query))
 	check_result(R);
       Result = false;
       break;
@@ -1291,11 +1368,7 @@ void pqxx::connection_base::EndCopyWrite()
 	"from PQputCopyEnd()");
   }
 
-  const result R(PQgetResult(m_Conn),
-		protocol_version(),
-		"[END COPY]",
-		encoding_code());
-  check_result(R);
+  check_result(make_result(PQgetResult(m_Conn), "[END COPY]"));
 
 #else
   WriteCopyLine(theWriteTerminator);
@@ -1322,6 +1395,12 @@ pqxx::internal::pq::PGresult *pqxx::connection_base::get_result()
 }
 
 
+void pqxx::connection_base::add_reactivation_avoidance_count(int n)
+{
+  m_reactivation_avoidance.add(n);
+}
+
+
 string pqxx::connection_base::esc(const char str[], size_t maxlen)
 {
   string escaped;
@@ -1348,8 +1427,8 @@ string pqxx::connection_base::esc(const char str[], size_t maxlen)
 
 #elif defined(PQXX_HAVE_PQESCAPESTRING)
   scoped_array<char> buf(new char[2*maxlen+1]);
-  const size_t bytes = PQescapeString(buf.c_ptr(), str, maxlen);
-  escaped.assign(buf.c_ptr(), bytes);
+  const size_t bytes = PQescapeString(buf.get(), str, maxlen);
+  escaped.assign(buf.get(), bytes);
 
 #else
   // Last-ditch workaround.  This has serious problems with multibyte encodings.
@@ -1404,7 +1483,7 @@ string pqxx::connection_base::esc_raw(const unsigned char str[], size_t len)
   if (!m_Conn) activate();
 
   PQAlloc<unsigned char> buf( PQescapeByteaConn(m_Conn, str, len, &bytes) );
-  if (!buf.c_ptr())
+  if (!buf.get())
   {
     // TODO: Distinguish different errors in exception type
     throw failure(ErrMsg());
@@ -1412,9 +1491,32 @@ string pqxx::connection_base::esc_raw(const unsigned char str[], size_t len)
 #else
   unsigned char *const s = const_cast<unsigned char *>(str);
   PQAlloc<unsigned char> buf( PQescapeBytea(s, len, &bytes) );
-  if (!buf.c_ptr()) throw bad_alloc();
+  if (!buf.get()) throw bad_alloc();
 #endif
-  return string(reinterpret_cast<char *>(buf.c_ptr()));
+  return string(reinterpret_cast<char *>(buf.get()));
+}
+
+
+pqxx::internal::reactivation_avoidance_exemption::
+  reactivation_avoidance_exemption(
+	connection_base &C) :
+  m_home(C),
+  m_count(gate::connection_reactivation_avoidance_exemption(C).get_counter()),
+  m_open(C.is_open())
+{
+  gate::connection_reactivation_avoidance_exemption gate(C);
+  gate.clear_counter();
+}
+
+
+pqxx::internal::reactivation_avoidance_exemption::
+  ~reactivation_avoidance_exemption()
+{
+  // Don't leave the connection open if reactivation avoidance is in effect and
+  // the connection needed to be reactivated temporarily.
+  if (m_count && !m_open) m_home.deactivate();
+  gate::connection_reactivation_avoidance_exemption gate(m_home);
+  gate.add_counter(m_count);
 }
 
 
@@ -1511,10 +1613,11 @@ void pqxx::connection_base::read_capabilities() throw ()
   {
     // Estimate server version by querying 'version()'.  This may not be exact!
     const string VQ = "SELECT version()";
-    const result r(PQexec(m_Conn, VQ.c_str()),
-		protocol_version(),
-		VQ,
-		encoding_code());
+    const result r = gate::result_creation::create(
+	PQexec(m_Conn, VQ.c_str()),
+	protocol_version(),
+	VQ,
+	encoding_code());
 
     int x=0, y=0, z=0;
     if ((sscanf(r[0][0].c_str(), "PostgreSQL %d.%d.%d", &x, &y, &z) == 3) &&
@@ -1529,23 +1632,34 @@ void pqxx::connection_base::read_capabilities() throw ()
 #endif
 
   const int v = m_serverversion;
+#if defined(PQXX_HAVE_PQPREPARE) || defined(PQXX_HAVE_PQFTABLECOL)
+  const int p = protocol_version();
+#endif
 
   m_caps[cap_prepared_statements] = (v >= 70300);
+
+#ifdef PQXX_HAVE_PQPREPARE
+  m_caps[cap_statement_varargs] = (v >= 70300 && (p >= 3));
+  m_caps[cap_prepare_unnamed_statement] = (p >= 3);
+#else
+  m_caps[cap_statement_varargs] = false;
+  m_caps[cap_prepare_unnamed_statement] = false;
+#endif
+
   m_caps[cap_cursor_scroll] = (v >= 70400);
   m_caps[cap_cursor_with_hold] = (v >= 70400);
+  m_caps[cap_cursor_fetch_0] = (v >= 70400);
   m_caps[cap_nested_transactions] = (v >= 80000);
   m_caps[cap_create_table_with_oids] = (v >= 80000);
+  m_caps[cap_read_only_transactions] = (v >= 80000);
 
 #ifdef PQXX_HAVE_PQFTABLECOL
-  const int p = protocol_version();
   m_caps[cap_table_column] = (p >= 3);
 #endif
-}
 
-
-void pqxx::connection_base::set_capability(capability c) throw ()
-{
-  m_caps[c] = true;
+#ifdef PQXX_HAVE_PQEXECPARAMS
+  m_caps[cap_parameterized_statements] = (p >= 3);
+#endif
 }
 
 
@@ -1566,3 +1680,40 @@ int pqxx::connection_base::encoding_code() throw ()
 #endif
 }
 
+
+#ifdef PQXX_HAVE_PQEXECPARAMS
+pqxx::result pqxx::connection_base::parameterized_exec(
+	const PGSTD::string &query,
+	const char *const params[],
+	const int paramlengths[],
+	int nparams)
+{
+  if (!supports(cap_parameterized_statements)) throw feature_not_supported(
+	"Database backend version does not support parameterized statements.");
+
+  result r = make_result(
+  	PQexecParams(
+		m_Conn,
+		query.c_str(),
+		nparams,
+		NULL,
+		params,
+		paramlengths,
+		NULL,
+		0),
+	query);
+  check_result(r);
+  get_notifs();
+  return r;
+}
+#else
+pqxx::result pqxx::connection_base::parameterized_exec(
+	const PGSTD::string &,
+	const char *const[],
+	const int[],
+	int)
+{
+  throw feature_not_supported(
+	"This libpq version does not support parameterized statements.");
+}
+#endif // PQXX_HAVE_PQEXECPARAMS
