@@ -7,7 +7,7 @@
  *      implementation of the pqxx::result class and support classes.
  *   pqxx::result represents the set of result tuples from a database query
  *
- * Copyright (c) 2001-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2010, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -53,7 +53,8 @@ pqxx::internal::result_data::result_data(pqxx::internal::pq::PGresult *d,
 pqxx::internal::result_data::~result_data() { PQclear(data); }
 
 
-void pqxx::internal::freemem_result_data(result_data *d) throw () { delete d; }
+void pqxx::internal::freemem_result_data(const result_data *d) throw ()
+	{ delete d; }
 
 
 pqxx::result::result(pqxx::internal::pq::PGresult *rhs,
@@ -90,10 +91,16 @@ void pqxx::result::tuple::swap(tuple &rhs) throw ()
 {
   const result *const h(m_Home);
   const result::size_type i(m_Index);
+  const size_type b(m_Begin);
+  const size_type e(m_End);
   m_Home = rhs.m_Home;
   m_Index = rhs.m_Index;
+  m_Begin = rhs.m_Begin;
+  m_End = rhs.m_End;
   rhs.m_Home = h;
   rhs.m_Index = i;
+  rhs.m_Begin = b;
+  rhs.m_End = e;
 }
 
 
@@ -124,8 +131,8 @@ bool pqxx::result::empty() const throw ()
 void pqxx::result::swap(result &rhs) throw ()
 {
   super::swap(rhs);
-  m_data = (c_ptr() ? c_ptr()->data : 0);
-  rhs.m_data = (rhs.c_ptr() ? rhs.c_ptr()->data : 0);
+  m_data = (get() ? get()->data : 0);
+  rhs.m_data = (rhs.get() ? rhs.get()->data : 0);
 }
 
 
@@ -257,7 +264,7 @@ const char *pqxx::result::CmdStatus() const throw ()
 
 const string &pqxx::result::query() const throw ()
 {
-  return c_ptr() ? c_ptr()->query : s_empty_string;
+  return get() ? get()->query : s_empty_string;
 }
 
 
@@ -292,7 +299,7 @@ bool pqxx::result::GetIsNull(pqxx::result::size_type Row,
 
 pqxx::result::field::size_type
 pqxx::result::GetLength(pqxx::result::size_type Row,
-                        pqxx::result::tuple::size_type Col) const
+                        pqxx::result::tuple::size_type Col) const throw ()
 {
   return PQgetlength(m_data, int(Row), int(Col));
 }
@@ -342,7 +349,7 @@ pqxx::result::table_column(tuple::size_type ColNum) const
     throw range_error("Invalid column index in table_column(): " +
       to_string(ColNum));
 
-  if (!c_ptr() || c_ptr()->protocol < 3)
+  if (!get() || get()->protocol < 3)
     throw feature_not_supported("Backend version does not support querying of "
       "column's original number",
       "[TABLE_COLUMN]");
@@ -368,19 +375,9 @@ int pqxx::result::errorposition() const throw ()
 
 // tuple
 
-pqxx::result::field pqxx::result::tuple::operator[](const char f[]) const
-{
-  return field(*this, m_Home->column_number(f));
-}
-
-
 pqxx::result::field pqxx::result::tuple::at(const char f[]) const
 {
-  const int fnum = m_Home->column_number(f);
-  // TODO: Should this be an out_of_range?
-  if (fnum == -1) throw argument_error(string("Unknown field '") + f + "'");
-
-  return field(*this, fnum);
+  return field(*this, m_Begin + column_number(f));
 }
 
 
@@ -412,6 +409,24 @@ pqxx::result::tuple::size_type pqxx::result::columns() const throw ()
 }
 
 
+pqxx::result::tuple::size_type
+pqxx::result::tuple::column_number(const char ColName[]) const
+{
+  const size_type n = m_Home->column_number(ColName);
+  if (n >= m_End)
+    return result().column_number(ColName);
+  if (n >= m_Begin)
+    return n - m_Begin;
+
+  const char *const AdaptedColName = m_Home->column_name(n);
+  for (size_type i = m_Begin; i < m_End; ++i)
+    if (strcmp(AdaptedColName, m_Home->column_name(i)) == 0)
+      return i - m_Begin;
+
+  return result().column_number(ColName);
+}
+
+
 pqxx::result::tuple::size_type
 pqxx::result::column_number(const char ColName[]) const
 {
@@ -424,6 +439,25 @@ pqxx::result::column_number(const char ColName[]) const
 }
 
 
+pqxx::result::tuple
+pqxx::result::tuple::slice(size_type Begin, size_type End) const
+{
+  if (Begin > End || End > size())
+    throw range_error("Invalid field range");
+
+  tuple result(*this);
+  result.m_Begin = m_Begin + Begin;
+  result.m_End = m_Begin + End;
+  return result;
+}
+
+
+bool pqxx::result::tuple::empty() const throw ()
+{
+  return m_Begin == m_End;
+}
+
+
 // const_iterator
 
 pqxx::result::const_iterator
