@@ -27,7 +27,7 @@ public:
     m_Done = true;
     PQXX_CHECK_EQUAL(
 	be_pid,
-	Conn().backendpid(),
+	conn().backendpid(),
 	"Notification came from wrong backend process.");
 
     cout << "Received notification: " << name() << " pid=" << be_pid << endl;
@@ -65,7 +65,7 @@ public:
 };
 
 
-void test_023(connection_base &, transaction_base &)
+void test_023(transaction_base &)
 {
   lazyconnection C;
   cout << "Adding listener..." << endl;
